// Vertex Shader  file "basic_phong.vert"	

#version 330

//Blinn-Phong Shading per Pixel for Multiple Light Sources

#define MAX_LIGHTS		3

//**********************************************************//
// Geometry uniforms
//**********************************************************//
uniform mat4 modelViewMatrix;
uniform mat3 normalMatrix;

//**********************************************************//
// Camera uniforms
//**********************************************************//
uniform mat4 projectionMatrix;
uniform vec3 cameraPosition;

//**********************************************************//
// Shadow mapping uniforms
//**********************************************************//
uniform mat4 pvmShadowMapping[MAX_LIGHTS];
uniform mat4 biasShadowMapping[MAX_LIGHTS];

//**********************************************************//
// Light uniforms
//**********************************************************//
uniform vec4 light_position[MAX_LIGHTS];
uniform int  light_counter;

//**********************************************************//
// Per vertex input attributes
//**********************************************************//
in vec4 vertex;		//in local coordinates
in vec3 normal;		//in normal coordinates
in vec3 color;		//in vertex color
in vec2 texure0;	//in texture coordinates

//**********************************************************//
// Variables for the pixe shader		
//**********************************************************//
out vec4 vertex_position;				//position of the observer
out vec3 light_vector[MAX_LIGHTS];		//direction fragment to light
out vec3 half_vector[MAX_LIGHTS];		//half vector for lighting
out vec3 frag_color;					//surface color
out vec3 frag_normal;					//normal vector of fragment
out vec3 view_dir;						//direction fragment to camera
out vec2 tex_coord0;					//texture coordinates
out vec4 shadow_coord[MAX_LIGHTS];		//shadow coordinates for texture lookup

//**********************************************************//
// Main program
//**********************************************************//
void main(void) {
	vertex_position = modelViewMatrix * vertex;	//vertex position

	//**********************************************************//
	// Light calculation
	//**********************************************************//
	frag_normal = normalize(normalMatrix * normal);		//transformed normal

	for(int i=0; i<light_counter; i++)	{
		//**********************************************************//
		// Spot lights
		//**********************************************************//
		if(light_position[i].w == 1)	{
			//light_vector[i] = ((modelViewMatrix * vec4(light_position[i].xyz, 0.0)) - vertex_position).xyz;
			light_vector[i] = light_position[i].xyz - vertex.xyz;

			half_vector[i] = normalize(normalize(cameraPosition - vertex.xyz) + normalize(light_vector[i]));

			view_dir = (vertex.xyz - cameraPosition);
		}
		//**********************************************************//
		// Directional light
		//**********************************************************//
		else	{
			//light_vector[i] = normalize(-light_position[i].xyz);
			light_vector[i] = -light_position[i].xyz;

			view_dir = vec3(normalize(vertex_position * -1));
		}

		//**********************************************************//
		// Coordinate for look up in the shadow map
		//**********************************************************//
		shadow_coord[i] = biasShadowMapping[i] * pvmShadowMapping[i] * vertex;	
	}

	//**********************************************************//
	// Color for fragment shader
	//**********************************************************//
	frag_color = color;	

	//**********************************************************//
	// Texture coordinates
	//**********************************************************//
	tex_coord0 = vec2(texure0);

	//**********************************************************//
	// Transformed position
	//**********************************************************//
	gl_Position = projectionMatrix * vertex_position;	
}
